@@ -2,7 +2,7 @@
 #--
 # This file is part of Sonic Pi: http://sonic-pi.net
 # Full project source: https://github.com/samaaron/sonic-pi
-# License: https://github.com/samaaron/sonic-pi/blob/master/LICENSE.md
+# License: https://github.com/samaaron/sonic-pi/blob/main/LICENSE.md
 #
 # Copyright 2013, 2014, 2015, 2016 by Sam Aaron (http://sam.aaron.name).
 # All rights reserved.
@@ -47,32 +47,9 @@ os = case RUBY_PLATFORM
        RUBY_PLATFORM
      end
 
-# special case for proctable lib
-sys_proctable_os = case os
-                   when :raspberry
-                     "linux"
-                   when :linux
-                     "linux"
-                   when :windows
-                     "windows"
-                   when :osx
-                     "darwin"
-                   end
-$:.unshift "#{File.expand_path("../vendor", __FILE__)}/sys-proctable-1.1.3/lib/#{sys_proctable_os}"
-
 
 $:.unshift "#{File.expand_path("../rb-native", __FILE__)}/#{ruby_api}/"
 
-
-## Add aubio native library to ENV if not present (the aubio library needs to be told the location)
-native_lib_path = File.expand_path("../../native/", __FILE__)
-ENV["AUBIO_LIB"] ||= Dir[native_lib_path + "/lib/libaubio*.{*dylib,so*,dll}"].first
-
-
-
-
-
-
 module SonicPi
   module Core
     class ThreadLocal
@@ -150,8 +127,57 @@ end
 module SonicPi
   module Core
     module SPRand
+      # we're limited to thread safe types so using ints
+      NOISE_TYPES = {
+        0 => :white,
+        1 => :pink,
+        2 => :light_pink,
+        3 => :dark_pink,
+        4 => :perlin
+      }
+
+      # define this helper first so we can set the constants below
+      def self.wav_from_buffer_file(filename)
+        ::WaveFile::Reader.new(
+          File.expand_path("/usr/share/sonic-pi/buffers/#{filename}", __FILE__),
+          ::WaveFile::Format.new(:mono, :float, 44100)
+        ).read(441000).samples.freeze
+      end
+
       # Read in same random numbers as server for random stream sync
-      @@random_numbers = ::WaveFile::Reader.new(File.expand_path("/usr/share/sonic-pi/buffers/rand-stream.wav", __FILE__), ::WaveFile::Format.new(:mono, :float, 44100)).read(441000).samples.freeze
+      # memoize for performance
+      RANDOM_NUMBERS_WHITE = self.wav_from_buffer_file("rand-stream.wav")
+      RANDOM_NUMBERS_PINK = self.wav_from_buffer_file("rand-stream-pink.wav")
+      RANDOM_NUMBERS_LIGHT_PINK = self.wav_from_buffer_file("rand-stream-light-pink.wav")
+      RANDOM_NUMBERS_DARK_PINK = self.wav_from_buffer_file("rand-stream-dark-pink.wav")
+      RANDOM_NUMBERS_PERLIN = self.wav_from_buffer_file("rand-stream-perlin.wav")
+
+      def self.get_random_number_distribution
+        type_idx = __thread_locals.get :sonic_pi_spider_random_gen_type
+        NOISE_TYPES[type_idx]
+      end
+
+      def self.set_random_number_distribution!(noise_type)
+        __thread_locals.set :sonic_pi_spider_random_gen_type, NOISE_TYPES.key(noise_type.to_sym)
+      end
+
+      def self.random_numbers
+        idx = __thread_locals.get :sonic_pi_spider_random_gen_type
+        case NOISE_TYPES[idx]
+        when :perlin
+          RANDOM_NUMBERS_PERLIN
+        when :pink
+          RANDOM_NUMBERS_PINK
+        when :light_pink
+          RANDOM_NUMBERS_LIGHT_PINK
+        when :dark_pink
+          RANDOM_NUMBERS_DARK_PINK
+        when :white
+          RANDOM_NUMBERS_WHITE
+        else
+          RANDOM_NUMBERS_WHITE
+        end
+      end
 
       def self.tl_seed_map(seed, idx=0)
         {:sonic_pi_spider_random_gen_seed => seed,
@@ -165,7 +191,7 @@ module SonicPi
       end
 
       def self.to_a
-        @@random_numbers
+        random_numbers
       end
 
       def self.inc_idx!(increment=1, init=0)
@@ -220,7 +246,7 @@ module SonicPi
         # also, scsynth server seems to swallow first rand
         # so always add 1 to index
         idx = (idx + 1) % 441000
-        @@random_numbers[idx] * max
+        random_numbers[idx] * max
       end
 
       def self.rand_i!(max, idx=nil)
@@ -336,7 +362,8 @@ module SonicPi
       attr_reader :map
 
       def initialize(*args)
-        @map = Hash[*args]
+        # let SPMaps be initialised from existing SPMaps and also standard Hashes.
+        @map = args.length == 1 && args.first.is_a?(self.class) ? args.first.map : Hash[*args]
         @map.freeze
         res = @map.all? {|k, v| k.sp_thread_safe? && v.sp_thread_safe?}
         @sp_thread_safe = !!res
@@ -800,7 +827,7 @@ module SonicPi
 
       def take_last(n=1)
         self if n >= size
-        self[(size-n)..-1]
+        self[(size-n)..size-1]
       end
 
       def butlast
